#include <avr/io.h>
#define F_CPU 1000000UL
#include <util/delay.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include "pin_macros.h"

#define ENCA 3
#define ENCB 4

#define Aout 0
#define Bout 1

#define reset 2



// Values returned by 'state'
#define DIR_NONE 0x0
// Clockwise step.
#define DIR_CW 0x10
// Anti-clockwise step.
#define DIR_CCW 0x20

#define R_START 0x0

#define HALF_STEP //fullstep is wrong for the small encoders and it doesnt matter for the opticals =D
#ifdef HALF_STEP
// Use the half-step state table (emits a code at 00 and 11)
#define R_CCW_BEGIN 0x1
#define R_CW_BEGIN 0x2
#define R_START_M 0x3
#define R_CW_BEGIN_M 0x4
#define R_CCW_BEGIN_M 0x5
const unsigned char ttable[6][4] = {
  // R_START (00)
  {R_START_M,            R_CW_BEGIN,     R_CCW_BEGIN,  R_START},
  // R_CCW_BEGIN
  {R_START_M | DIR_CCW, R_START,        R_CCW_BEGIN,  R_START},
  // R_CW_BEGIN
  {R_START_M | DIR_CW,  R_CW_BEGIN,     R_START,      R_START},
  // R_START_M (11)
  {R_START_M,            R_CCW_BEGIN_M,  R_CW_BEGIN_M, R_START},
  // R_CW_BEGIN_M
  {R_START_M,            R_START_M,      R_CW_BEGIN_M, R_START | DIR_CW},
  // R_CCW_BEGIN_M
  {R_START_M,            R_CCW_BEGIN_M,  R_START_M,    R_START | DIR_CCW},
};
#else
// Use the full-step state table (emits a code at 00 only)
#define R_CW_FINAL 0x1
#define R_CW_BEGIN 0x2
#define R_CW_NEXT 0x3
#define R_CCW_BEGIN 0x4
#define R_CCW_FINAL 0x5
#define R_CCW_NEXT 0x6

const unsigned char ttable[7][4] = {
  // R_START
  {R_START,    R_CW_BEGIN,  R_CCW_BEGIN, R_START},
  // R_CW_FINAL
  {R_CW_NEXT,  R_START,     R_CW_FINAL,  R_START | DIR_CW},
  // R_CW_BEGIN
  {R_CW_NEXT,  R_CW_BEGIN,  R_START,     R_START},
  // R_CW_NEXT
  {R_CW_NEXT,  R_CW_BEGIN,  R_CW_FINAL,  R_START},
  // R_CCW_BEGIN
  {R_CCW_NEXT, R_START,     R_CCW_BEGIN, R_START},
  // R_CCW_FINAL
  {R_CCW_NEXT, R_CCW_FINAL, R_START,     R_START | DIR_CCW},
  // R_CCW_NEXT
  {R_CCW_NEXT, R_CCW_FINAL, R_CCW_BEGIN, R_START},
};
#endif

int main(void) {

  int listening = 1;
  unsigned char state = R_START;
  unsigned char dir;

DDRB = 0b00000011;
PORTB = 0b00011100;

while (1) {


    if(bit_is_clear(PINB,reset))
    {
        if((bit_is_set(PINB,Aout))||(bit_is_set(PINB,Bout)))
        {
            b0_low;
            b1_low;
        }
      state = R_START;  
      listening = 1;
    }
    else
    {
      if(listening==1)
      {
        int A = 0;
        int B = 0;
        if(bit_is_set(PINB,ENCA)) A = 1;
        if(bit_is_set(PINB,ENCB)) B = 1;  

  unsigned char pinstate = (A << 1) | B;
  // Determine new state from the pins and state table.
  state = ttable[state & 0xf][pinstate];
  dir = state & 0x30;

   if(dir == DIR_CW){b0_high;   listening = 0;}
   else if (dir == DIR_CCW){b1_high;listening = 0;}

  }
    
 }


    

}

return 0;
}


